
local map = require"map"
local bullet = require"bullet"
local snake = require"iasnake"
local play = require"play"
local Item = require"items"

-- --------------------------------------------------------------------------------
-- local shader = love.graphics.newShader
-- [[
--   extern Image bgimg;
--
--   vec4 effect(vec4 clr, Image img, vec2 imgpos, vec2 scrpos)
--   {
--     float alpha = Texel(img, imgpos).w * clr.w;
--     vec4 bgpixel = Texel(bgimg, vec2(scrpos.x / love_ScreenSize.x, 1. - scrpos.y/love_ScreenSize.y));
--     return vec4(vec3(1., 1., 1.) - bgpixel.xyz, alpha);
--   }
-- ]]
--------------------------------------------------------------------------------
local PS = class:extend()
PS.new = function(self, img, x, y, l, c)
  self.cor = c or {255,255,255}
  self.t = timer.after( l, function() self.p = {} self.isDead = true end)
  self.x = x or 0
  self.y = y or 0
	self.i = img
	self.p = {}
  for i=1, 25 do
    local a = math.rad( math.random(360))
    local s = math.random( 5, 15)*10
    local vx = math.cos( a) * s
    local vy = math.sin( a) * s
    self.p[i] = {x=x, y=y, vx=vx, vy=vy, l=math.random(1,l)}
  end
end
PS.update = function( self, dt)
  for i=1, #self.p do
    local p = self.p[i]
    p.x = p.x + p.vx*dt
    p.y = p.y + p.vy*dt
    p.l = p.l - 5*dt
  end
end
PS.draw = function( self)
  for i=1, #self.p do
    local p = self.p[i]
    love.graphics.setColor(self.cor[1], self.cor[2], self.cor[3], math.min(p.l * 255, 255))
    love.graphics.draw( self.i, p.x, p.y)
  end
end
--------------------------------------------------------------------------------
--[[
       ██   ██ ██    ██ ██████
       ██   ██ ██    ██ ██   ██
       ███████ ██    ██ ██   ██
       ██   ██ ██    ██ ██   ██
██     ██   ██  ██████  ██████
--]]

local numbers_font = love.graphics.newImageFont("img/numbers.png", "1234567890")
local small_font = love.graphics.newImageFont("img/numbers_small.png", "1234567890")
local hud = {}
hud.start = function( self)
  local x,y = love.graphics.getDimensions( )
  self.score_board_pos = vec( x/2 - 80, 0)
  self.score_board = love.graphics.newCanvas(160, 22)
  love.graphics.setCanvas( self.score_board)
  love.graphics.setColor(85, 0, 70)
  love.graphics.rectangle("fill", 0, 0, 160, 22)
  love.graphics.setColor(10, 22, 95)
  love.graphics.rectangle("fill", 1, 0, 158, 28)
  love.graphics.setColor(35, 65, 175)
  love.graphics.rectangle("fill", 2, 0, 156, 27)
  love.graphics.setCanvas()

  local p = (self.score_board_pos / game.tile):vfloor()
  self.crystals = {}
  self.slot = {}
  for i=1,3 do
    local di = (i-1.5)/2
    self.slot[i] = Item["frame"](p.x+0.5, di)
    self.slot[i]:setScale(0.5)
    self.crystals[i] = Item["crystal"](p.x+0.5, di)
    self.crystals[i]:setScale(0.5)
  end
end
hud.draw = function( self, score)
  love.graphics.setColor(255,255,255)
  love.graphics.draw( self.score_board, self.score_board_pos.x, self.score_board_pos.y)
  love.graphics.setFont( numbers_font)
  love.graphics.printf( score, self.score_board_pos.x, self.score_board_pos.y, 156, "center")
  for i=1,3 do
    self.slot[i]:draw()
    if i<=game.player.crystal_counter then
      self.crystals[i]:draw()
    end
  end
end
--------------------------------------------------------------------------------
--[[
 ██████   █████  ███    ███ ███████
██       ██   ██ ████  ████ ██
██   ███ ███████ ██ ████ ██ █████
██    ██ ██   ██ ██  ██  ██ ██
 ██████  ██   ██ ██      ██ ███████
--]]
local exit, grid, cnv_grid
local game = require"data"

game.init = function( self)
  self.tile = 16
  self.score = 0
  self.itemSpawnTimer = 0
  self.itemSpawnRate = 1
  self.items = {}
end
game.enter = function( self, from)
  love.audio.stop( )
  -- collider = HC(64, on_collision, collision_stop)
  self.map= map( math.random(28,62), math.random(26,32))

  self.bh= bullet()
  local snek = from and from.player
  self:restart(snek)
  self.snakes = { self.player}
  self.item_count = { items = {}, ["spawner"] = 0}

  local mw,mh = self.map:getDimensions( )
  local w,h = self.tile *mw, self.tile *mh

  grid = love.graphics.newCanvas(w, h)
  self.canvas = love.graphics.newCanvas(w, h)
  self.mapCanvas = love.graphics.newCanvas(w, h)
  love.graphics.setBackgroundColor( self.colors.bg[ math.random( #self.colors.bg)])
  love.window.setMode( w, h )
  hud:start()
end
game.over = function( self)
    Gamestate.push( gameover)
end
game.restart = function( self, snek)
    timer.clear()
    self.items = {}
    self.ps = {}
    exit = false
    self.player = snek or play( 1, 1)
    self.player:setSpeed(0.5, 2)
    self.player.crystal_counter = 0
    self.score = 0
    self.dif = 2
    self.difMod = 0
end
game.next_please = function( )
    Gamestate.push( warp)
end

game.setCanvas = function( self)
  love.graphics.setColor(255,255,255)

  if self.map.refresh then
    if not cnv_grid then
      love.graphics.setCanvas( grid)

      love.graphics.setColor( 255,255,255, 25)
      for x=0,self.map.d.x-1 do
        for y=0,self.map.d.y-1 do
          love.graphics.rectangle( "line", x*self.tile, y*self.tile, self.tile, self.tile)
        end
      end
      love.graphics.setCanvas( )
      cnv_grid = true
    end
    love.graphics.setCanvas( self.mapCanvas)
      self.map:draw()
    love.graphics.setCanvas( )
    self.map.refresh = false
  end

  love.graphics.setCanvas( self.canvas)
  love.graphics.clear()
  hud:draw( self.player.points)


  love.graphics.setFont( small_font)
  for i=1,#self.items do
    self.items[i]:draw()
  end
  for i=1,#self.snakes do
    self.snakes[i]:draw()
  end
  for i=1,#self.ps do
    self.ps[i]:draw( )
  end
  self.bh:draw()
  love.graphics.setCanvas( )
end

game.addSnek = function( self, x, y, d)
  table.insert( self.snakes, snake( x, y, math.random(4, 8), d))
end
game.addItem = function( self, itemName, ...)
  table.insert( self.items, Item[ itemName]( ...))
end
game.itemCounter = function( self, itemName, number)
  local n = number or 1
  if not self.item_count[itemName] then
    table.insert(self.item_count.items, itemName)
    self.item_count[itemName] = n
  else
    self.item_count[itemName] = self.item_count[itemName] + n
  end
end

game.shoot = function( self, x, y, d)
  self.bh:action( x, y, d, 8, 0.5)
end
game.sparks = function( self, img, x, y, l, c)
  table.insert(self.ps, PS( game.img.parts[img], x, y, l, c))
end
game.getFreeSpot = function( self)

  local p = self.map:getRandomTile()
  if self.snakes then
    for i,snek in ipairs(self.snakes) do
      while p == snek.gp or snek:checkTailPos(p.x, p.y) do
        p = self.map:getRandomTile()
      end
    end
  end
  return p
end
game.getSpawnPoint = function( self, b)
  local gp
  local safe = false
  local c = function( gp)
    for i=-b,b do
      for j=-b,b do
        local gpp = gp + vec(i,j)
        if self.map:isSolid(gpp.x, gpp.y) then
          return false
        end
      end
    end
    return true
  end
  while not safe do
    gp = self:getFreeSpot()
    safe = c( gp)
  end
  return gp
end

--[[
██    ██ ██████  ██████   █████  ████████ ███████
██    ██ ██   ██ ██   ██ ██   ██    ██    ██
██    ██ ██████  ██   ██ ███████    ██    █████
██    ██ ██      ██   ██ ██   ██    ██    ██
 ██████  ██      ██████  ██   ██    ██    ███████
--]]
game.update = function( self, dt)

  self.itemSpawnTimer = self.itemSpawnTimer + dt
  if self.itemSpawnTimer > self.itemSpawnRate then
    local p = game:getFreeSpot()
    local it = Item.getRndItem()
    table.insert( self.items, it( p.x, p.y))
    self.itemSpawnTimer = 0
    self.itemSpawnRate = math.random(2, 5)
  end
  self.bh:update(dt)

  for i=1,#self.items do
    local item = self.items[i]
    item:update( dt)
    for i,snek in ipairs( self.snakes) do
      if snek:sameSpot( item.gp) then
        item:action( snek)
      end
    end
    self.bh:loop( function( b)
      if b:checkColl( item.p.x, item.p.y, 16, 16) then
        b:hit( item)
      end
    end)
  end

  for i=1,#self.snakes do
    local snek = self.snakes[i]
    snek:update( dt)
    for a,s in ipairs( self.snakes) do
      if i~=a then
        snek:bite(s)
      end
    end
    self.bh:loop( function( b)
      local sb = snek:checkColl( b.p.x, b.p.y, 6, 6)
      if sb then
        b:hit( sb)
      end
    end)
  end

  if not exit and self.player.crystal_counter >=3 then
    p = self.map:getCenter()
    exit = true
    self:addItem('nextlevel', p.x, p.y)
  end
  if self.player.points-self.difMod >= 1000 then
    self.dif = self.dif+1
    self.difMod = self.player.points
  end
  if #self.snakes<self.dif then
    local sp = self.item_count["spawner"]
    if sp <=self.dif-2 and math.random(300) == 33 then
      p = self:getSpawnPoint(1)
      self:addItem('spawner', p.x, p.y)
    end
  end

  for i=1 ,#self.ps do
    self.ps[i]:update( dt)
  end

  self.bh:loop( function( b)
    for i=1,#self.map.cells do
      local block = self.map.cells[i]
      local border_hit = ( b.gp.x < 1 or b.gp.y < 1 or b.gp.x > (game.map.d.x - 2) or b.gp.y > (game.map.d.y - 2))

      if not border_hit then
        if b:checkColl( block.p.x, block.p.y, 16, 16) then
          b:hit( )
        end
      else
        if not b.isDead and border_hit then
          b:hit( )
        end
      end
    end
  end)
  -------------------------------------------cemetery
  for i,snek in ipairs( self.snakes) do
    if snek.isDead then
      table.remove( self.snakes, i)
    end
  end
  for i,item in ipairs( self.items) do
    if item.isDead then
      table.remove( self.items, i)
    end
  end
  for i,spark in ipairs( self.ps) do
    if spark.isDead then
      table.remove( self.ps, i)
    end
  end
  if self.player.isDead then
    self.snd.death:play()
    self.score = self.player.points
    timer.after( 1, function() self:over() end)
  end

  -------------------------------------------canvas
  timer.update( dt)
  self:setCanvas( )
end
--[[
██████  ██████   █████  ██     ██
██   ██ ██   ██ ██   ██ ██     ██
██   ██ ██████  ███████ ██  █  ██
██   ██ ██   ██ ██   ██ ██ ███ ██
██████  ██   ██ ██   ██  ███ ███
--]]
game.draw = function( self)

  love.graphics.setColor(255,255,255)

  love.graphics.draw( grid)
  love.graphics.setColor(0,0,0,150)
  love.graphics.draw( self.mapCanvas,6,6)
  love.graphics.draw( self.canvas,6,6)
  love.graphics.setColor(255,255,255)
  love.graphics.draw( self.mapCanvas)
  love.graphics.draw( self.canvas)


end
game.keypressed = function( self, k)

  self.player:keypressed( k)
  if k == 'p' then
    Gamestate.push( pause)
  end
end
game.keyreleased = function( self, k)
  self.player:keyreleased( k)
end

-- game.mousepressed = function( self, x, y, k)
  -- local p = (vec(x,y)/self.tile):vfloor()
  -- self:addItem( "crystal", p.x, p.y)
-- end
return game
