local templates = {}
templates.center = {
  {
    {1,1,1,0,1,1,1},
    {1,0,0,0,0,0,1},
    {1,0,0,0,0,0,1},
    {0,0,0,0,0,0,0},
    {1,0,0,0,0,0,1},
    {1,0,0,0,0,0,1},
    {1,1,1,0,1,1,1},
  },
  {
    {1,0,0,1,1,0,0,1},
    {1,0,0,0,0,0,0,1},
    {1,0,1,0,0,1,0,1},
    {0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0},
    {1,0,1,0,0,1,0,1},
    {1,0,0,0,0,0,0,1},
    {1,0,0,1,1,0,0,1},
  },
  {
    {1,1,1,1,1,1,0,1},
    {0,0,0,0,0,0,0,1},
    {1,0,0,0,0,0,0,1},
    {1,0,0,0,0,0,0,1},
    {1,0,0,0,0,0,0,1},
    {1,0,0,0,0,0,0,1},
    {1,0,0,0,0,0,0,0},
    {1,0,1,1,1,1,1,1},
  },
  {
    {0,0,0,0,0,0,0,0,0},
    {1,0,1,0,1,0,1,0,1},
    {0,0,0,0,0,0,0,0,0},
    {1,0,1,0,0,0,1,0,1},
    {0,0,0,0,0,0,0,0,0},
    {1,0,1,0,0,0,1,0,1},
    {0,0,0,0,0,0,0,0,0},
    {1,0,1,0,1,0,1,0,1},
    {0,0,0,0,0,0,0,0,0},
  },
}
local cell = (require"cell"):extend()
cell.new = function( self, x, y, img)
  cell.super.new(self, x, y)
  self.i = img
end

local function putStructureTable( map, tab, x, y)
  local tw,th = #tab[1], #tab
  for tx=1,tw do
    for ty=1,th do
      local ht = (vec(tw,th)/2):vfloor()
      local x,y = (x-ht.x) +tx-1, (y-ht.y) +ty-1
      map[x][y] = tab[tx][ty]
    end
  end
end

local map = {}
map.__index = map
function map.new(w, h)
  local m = {
    d = vec( w, h),
    data = {},
    cells = {},
  }

  for x=0,w-1 do
    m.data[x] = {}
    for y=0,h-1 do
      local n = 0
      if x == 0 or x == w-1 or y == 0 or y == h-1 then
        n=1
      end
      m.data[x][y] = n

    end
  end

  m.img = 'block' .. math.random(4)
  m.color = game.colors.blocks[ math.random( #game.colors.blocks)]

  local cntr = templates.center[math.random(#templates.center)]
  putStructureTable( m.data, cntr, math.floor(w/2), math.floor(h/2))


  for x=0,w-1 do
    for y=0,h-1 do
      if m.data[x][y]==1 then
        table.insert( m.cells, cell( x, y, m.img))
      elseif m.data[x][y]==2 then
        timer.after(0.5, function()
          game:addItem( 'spawner', x, y)
        end)
      end


    end
  end

  m.refresh = true
  return setmetatable(m, map)
end
function map:draw( )
  love.graphics.setColor( self.color)
  for n,cell in ipairs(self.cells) do
    love.graphics.draw( game.img.misc[ cell.i], cell.p.x, cell.p.y)
  end
end
function map:isValid( x, y)
  if self.data[x] and self.data[x][y] then
    return true
  end
  return false
end
function map:make_air( x, y)
  self.refresh = true
  self.data[x][y] = 0
  for i=1,#self.cells do
    local cell = self.cells[i]
    if cell.gp.x==x and cell.gp.y==y then
      table.remove( self.cells, i)
      return
    end
  end
end
function map:make_solid( x, y)
  self.data[x][y] = 1
end
function map:block( x, y)
  self:make_solid( x, y)
  self.refresh = true
  table.insert( self.cells, cell( x, y, self.img))
end
function map:getDimensions( )
  return self.d.x, self.d.y
end
function map:getCenter( )
  return (self.d/2):vfloor()
end
function map:getRandomTile( )
    local x,y = math.random(1,self.d.x-1), math.random(1,self.d.y-1)
    while self:isSolid( x, y) do
      x,y = math.random(1,self.d.x-1), math.random(1,self.d.y-1)
    end
    return vec(x,y)
end
function map:isSolid( x, y)
  if not (self.data[x] and self.data[x][y]) then
      return false
  end
  return self.data[x][y] == 1
end

setmetatable(map, { __call = function(_, ...) return map.new(...) end })
return map
