local cell = require"cell"
local body = cell:extend()
body.new = function( self, x, y, l, r, i, sy, sx)
  body.super.new( self, x, y, l)
  self.r = r or 0
  -- self.l = l or 1
  self.d = vec(1,0)
  self.oldi = i or 'body1'
  self.i = i or 'body1'
  self.sx = sx or 1
  self.sy = sy or 1
end

local ghost = cell:extend()
ghost.new = function( self, x, y)
  ghost.super.new( self, x, y)
  self.i = 'ghost'
end

local snake = cell:extend()
snake.new = function( self, x, y, l)
  snake.super.new( self, x, y, 10)
  self.oldgp = vec()
  self.d = vec(1,0)
  self.dir = 'r'
  self.maxspd = 8
  self.spd = 1
  self.spdM = 0
  self.moverate = 0
  self.newrot = 0
  self.rot = 0
  self.oldRot = 0
  self.lengh = 6
  self.frm = 0
  self.firerate = 1
  self.nfr = 1
  self.firetime = 1
  self.canShoot = false
  self.tail = {}
  self.ghost = {}
  self.scale = 1
  self.color = {204, 61, 23}
  local g = anim8.newGrid(16, 16, 32, 16)
  self.ani = anim8.newAnimation(g('1-2',1), 0.1)
  for i=1, self.lengh do
    table.insert( self.tail, body( x, y, 2))
  end
end

snake.draw = function( self)
  love.graphics.setColor(self.color)

  for i=1,#self.tail do
    local k = self.tail[i]
    local img = i==#self.tail and 'tail' or k.i
    self.ani:draw( game.img.snake[img],  k.p.x+8, k.p.y+8, k.r, k.sx, k.sy, 8, 8)
  end

  self.ani:draw( game.img.snake.head, self.p.x+8, self.p.y+8, self.rot, 1,1, 8, 8)
end

snake.incMaxSpeed = function( self, s)
    self.maxspd = self.maxspd + s
end
snake.incDefaultFireRate = function( self, s)
    self.nfr = self.nfr + s
end
snake.setSpeed = function( self, s, t)
    self.spdM = self.spdM + 1
    self.spd = self.maxspd*s
    timer.after( t, function() self.spdM = self.spdM - 1 end)
end
snake.setFirerate = function( self, r, t)
    self.firerate = math.max(0.1, r)
    self.frm = self.frm +1
    timer.after( t, function() self.frm = self.frm - 1 end)
end
snake.shoot = function( self)
  if self.canShoot and self.firetime>= self.firerate then
    local p = (self.gp + self.d)*game.tile
    game:shoot( p.x+8, p.y+8, self.dir)
    game.snd.shoot:play()
    self.firetime = 0
  end
end
snake.checkTailPos = function( self, gp)
  for i=1,#self.tail do
    if self.tail[i].gp == gp then
      return true, i
    end
  end
  return false
end
snake.checkColl = function( self, x, y, w, h)
  local head = intersects( self.p.x, self.p.y, 16, 16, x, y, w, h)
  local tail, t_index = false,0
  for i=1,#self.tail do
    if intersects( self.tail[i].p.x, self.tail[i].p.y, 16, 16, x, y, w, h) then
      tail = true
      t_index = i
    end
  end
  return (head and self) or (tail and self.tail[t_index])
end
snake.checkTail = function( self, x, y)
  local gp = self.gp + vec( x, y)
  return self:checkTailPos( gp)
end
-- snake.damage = function( self, dmg)
--   self.hp = self.hp - dmg
--   if self.hp <= 0 then
--     self:die()
--   end
-- end
snake.increaseTail = function( self, gp)
  local p = gp or self.tail[self.lengh].oldgp
  table.insert( self.tail, body( p.x, p.y, 2))
end
snake.sameSpot = function( self, gp)
  return self.gp == gp
end
snake.grow = function( self, s)
  if s>0 then
    -- self.lengh = self.lengh + s
    table.insert( self.tail, body( self.oldgp.x, self.oldgp.y, 2))
    -- self.scale = 1.25
    self:incDefaultFireRate(-0.05)
    self:incMaxSpeed(-0.1)
  else
    if self.lengh-math.abs(s)>2 then
      for i=1,math.abs(s) do
        table.remove( self.tail, #self.tail)
      end
      self:incDefaultFireRate(0.05)
      self:incMaxSpeed(0.1)
    else
      self:die()
    end
  end
  self.lengh = #self.tail
end
snake.bite = function( self, other)
  local thit, tind = self:checkTailPos(other.gp)
  if self:sameSpot(other.gp) then
    if self.lengh <= other.lengh then
      self:die()
    else
      self:grow(3)
      self:coin(100)
    end
    game.snd.bite:play()
  elseif thit and not self.tail[tind].isDead then
    self.tail[tind]:die()
    other:grow(1)
    other:coin(50)
    game.snd.bite:play()
  end
end
snake.coin = function( self, other)
  self:grow(1)
end
snake.crystal = function( self, other)
  self:grow(2)
end
--[[
███    ███  ██████  ██    ██ ███████
████  ████ ██    ██ ██    ██ ██
██ ████ ██ ██    ██ ██    ██ █████
██  ██  ██ ██    ██  ██  ██  ██
██      ██  ██████    ████   ███████
--]]

snake.move = function( self, dt)
  self.moved = false
  self.moverate = self.moverate +self.spd*dt
  self.firetime = self.firetime +dt

  if self.spdM<=0 then
    if self.spd == 0 then
      self.spd = 0.2
    end
    if self.spd < self.maxspd then
      self.spd = math.min(self.maxspd, self.spd+self.spd*dt)
    else
      self.spd = math.max(self.maxspd, self.spd-self.spd*dt)
    end
  end
  if self.frm<=0 then
    if self.firerate < self.nfr then
      self.firerate = math.min(self.nfr, self.firerate+self.firerate*dt)
    else
      self.firerate = math.max(self.nfr, self.firerate-self.firerate*dt)
    end
  end
  if self.moverate >= 1 then
    self.rot = self.newrot
    local hitself = self:checkTail( self.d.x, self.d.y)
    if hitself or game.map:isSolid( self.gp.x, self.gp.y) then
      self:die()
      return
    end
    self.moved = true

    local sy, i = 1, 'body1'
    if self.oldRot ~= self.rot then
      i = 'body2'
      if self.rot > self.oldRot or math.deg(self.oldRot - self.rot) == 270 then
        sy = math.deg(self.rot-self.oldRot) == 270 and 1 or -1
      end
      self.oldRot = self.rot
    end
    --

    local p = self.gp + self.d
    self.oldgp = self.gp:clone()
    self:setPosition( p.x, p.y)
    self.moverate = 0

    local neck = self.tail[1]
    neck.oldgp = neck.gp:clone()
    neck:setPosition( self.oldgp.x, self.oldgp.y)
    neck.oldsy = neck.sy
    neck.sy = sy
    neck.oldr = neck.r
    neck.r = self.oldRot
    neck.oldi = neck.i
    neck.i = i

    for i=2, #self.tail do
      bd = self.tail[i]
      if bd then
        bd.oldgp = bd.gp:clone()
        local np = self.tail[i-1].oldgp
        bd:setPosition( np.x, np.y)
        bd.oldr = bd.r
        bd.r = self.tail[i-1].oldr
        bd.oldi = bd.i
        bd.i = self.tail[i-1].oldi
        bd.oldsy = bd.sy
        bd.sy = self.tail[i-1].oldsy

        if bd.hp <=0 then
          bd:die()
        end
      end
    end
    self.scale = 1

    for i,k in ipairs( self.tail) do
      if k.isDead then
        self:grow( -(#self.tail - i+1))
      end
    end
  end
end
-- snake.move = function( self, dt)  --OLD
--   self.moved = false
--   self.moverate = self.moverate +self.spd*dt
--   self.firetime = self.firetime +dt
--
--   if self.spdM<=0 then
--     if self.spd == 0 then
--       self.spd = 0.2
--     end
--     if self.spd < self.maxspd then
--       self.spd = math.min(self.maxspd, self.spd+self.spd*dt)
--     else
--       self.spd = math.max(self.maxspd, self.spd-self.spd*dt)
--     end
--   end
--   if self.moverate >= 1 then
--     local hitself = self:checkTail( self.d.x, self.d.y)
--     if hitself or game.map:isSolid( self.gp.x, self.gp.y) then
--       self:die()
--       return
--     end
--     self.moved = true
--     local sy, i = 1
--     if self.oldRot ~= self.rot then
--       i = 'body2'
--       if self.rot > self.oldRot or math.deg(self.oldRot - self.rot) == 270 then
--         sy = math.deg(self.rot-self.oldRot) == 270 and 1 or -1
--       end
--       self.oldRot = self.rot
--     end
--
--     table.insert( self.tail, 1, body( self.gp.x, self.gp.y, self.lengh, self.rot, i, sy*self.scale, self.scale))
--
--     local p = self.gp + self.d
--     self.oldgp = self.gp:clone()
--     self:setPosition( p.x, p.y)
--     self.moverate = 0
--     for i,k in ipairs(self.tail) do
--       k.l = k.l - 1
--       if k.hp <=0 then
--         self:grow(-(#self.tail - i))
--       end
--       if k.l <= 0 then
--         table.remove( self.tail, i)
--       end
--     end
--     self.scale = 1
--   end
-- end

snake.step = function( self)
  return self.moved
end

return snake
